<?php

/*
 * Plugin: Badwords Judge
 * ~~~~~~~~~~~~~~~~~~~~~~
 * For a detailed description and documentation, please refer to:
 * http://www.undef.name/XAseco1/Badwords-Judge.php
 *
 * ----------------------------------------------------------------------------------
 * Author:		undef.de
 * Version:		0.9.9
 * Date:		2013-11-26
 * Copyright:		2010 - 2013 by undef.de
 * System:		XAseco/1.16+
 * Game:		Trackmania United Forever (TMUF)
 * ----------------------------------------------------------------------------------
 *
 * LICENSE: This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * ----------------------------------------------------------------------------------
 *
 * Dependencies: none
 */

Aseco::registerEvent('onSync',				'bwj_onSync');
Aseco::registerEvent('onChat',				'bwj_onChat');
Aseco::registerEvent('onPlayerConnect',			'bwj_onPlayerConnect');
Aseco::registerEvent('onBillUpdated',			'bwj_onBillUpdated');

Aseco::addChatCommand('bwj',				'Admin command for BadwordsJudge', true);

global $bwj_config;

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function bwj_onSync ($aseco) {
	global $bwj_config;


	// Check for the right XAseco-Version
	$xaseco_min_version = '1.16';			// Official "1.15b", but not useable with version_compare()
	if ( defined('XASECO_VERSION') ) {
		$version = str_replace(
			array('a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i'),
			array('.1','.2','.3','.4','.5','.6','.7','.8','.9'),
			XASECO_VERSION
		);
		if ( version_compare($version, $xaseco_min_version, '<') ) {
			trigger_error('[plugin.badwords_judge.php] Not supported XAseco version ('. XASECO_VERSION .')! Please update to min. version '. $xaseco_min_version .'!', E_USER_ERROR);
		}
	}
	else {
		trigger_error('[plugin.badwords_judge.php] Can not identify the System, "XASECO_VERSION" is unset! This plugin runs only with XAseco/'. $xaseco_min_version .'+', E_USER_ERROR);
	}

	if ($aseco->server->getGame() != 'TMF') {
		trigger_error('[plugin.badwords_judge.php] This plugin supports only TMF, can not start with a "'. $aseco->server->getGame() .'" Dedicated-Server!', E_USER_ERROR);
	}

//	if (strtoupper($aseco->server->packmask) == 'STADIUM') {
//		trigger_error('[plugin.badwords_judge.php] This plugin supports a TMU Dedicated-Server only! You need to adjust <system_config><packmask> to "united" to have a dedicated TMU only Server.', E_USER_ERROR);
//	}

	if (!$bwj_config = $aseco->xml_parser->parseXML('badwords_judge.xml')) {
		trigger_error('[plugin.badwords_judge.php] Could not read/parse config file "badwords_judge.xml"!', E_USER_ERROR);
	}

	$bwj_config = $bwj_config['BADWORDS_JUDGE'];

	// Check for oversized <badword_punish_note>
	if (strlen($bwj_config['MESSAGES'][0]['BADWORD_PUNISH_NOTE'][0]) > 255) {
		trigger_error('[plugin.badwords_judge.php] <messages><badword_punish_note> in "badwords_judge.xml" is too large!', E_USER_ERROR);
	}

	$bwj_config['VERSION'] = '0.9.9';

	// Register this to the global version pool (for up-to-date checks)
	$aseco->plugin_versions[] = array(
		'plugin'	=> 'plugin.badwords_judge.php',
		'author'	=> 'undef.de',
		'version'	=> $bwj_config['VERSION']
	);


	// Transform 'TRUE' or 'FALSE' from string to boolean
	$bwj_config['ENABLED'][0]		= ((strtoupper($bwj_config['ENABLED'][0]) == 'TRUE') ? true : false);
	$bwj_config['MESSAGES_IN_WINDOW'][0]	= ((strtoupper($bwj_config['MESSAGES_IN_WINDOW'][0]) == 'TRUE') ? true : false);
	$bwj_config['IGNORE'][0]['OPERATOR'][0]		= ((strtoupper($bwj_config['IGNORE'][0]['OPERATOR'][0]) == 'TRUE') ? true : false);
	$bwj_config['IGNORE'][0]['ADMIN'][0]		= ((strtoupper($bwj_config['IGNORE'][0]['ADMIN'][0]) == 'TRUE') ? true : false);
	$bwj_config['IGNORE'][0]['MASTERADMIN'][0]	= ((strtoupper($bwj_config['IGNORE'][0]['MASTERADMIN'][0]) == 'TRUE') ? true : false);


	// Make sure the fine are (int)
	$bwj_config['PUNISH_TICKETS'][0]['FINE1'][0]	= (int)$bwj_config['PUNISH_TICKETS'][0]['FINE1'][0];
	$bwj_config['PUNISH_TICKETS'][0]['FINE2'][0]	= (int)$bwj_config['PUNISH_TICKETS'][0]['FINE2'][0];
	$bwj_config['PUNISH_TICKETS'][0]['FINE3'][0]	= (int)$bwj_config['PUNISH_TICKETS'][0]['FINE3'][0];
	$bwj_config['PUNISH_TICKETS'][0]['FINE4'][0]	= (int)$bwj_config['PUNISH_TICKETS'][0]['FINE4'][0];


	// Init Player-Array
	$bwj_config['PLAYERS']['STATE'] = array();
	$bwj_config['PLAYERS']['PUNISH'] = array();
	$bwj_config['PLAYERS']['BILLS'] = array();
	$bwj_config['NAME'][0] = '{#server}[$I$09FBadwords $I$FFF$WJugde$Z$S{#server}] ';


	// Build the REGEX pattern
	$tmp = array();
	foreach ($bwj_config['BADWORDS_LIST'][0]['BAD'] as $word) {
		$tmp[] = '\b'. $word .'\b';
	}
	$bwj_config['PATTERN'] = '/('. implode('|', $tmp) .')/i';

	unset($tmp);
	unset($bwj_config['BADWORDS_LIST'][0]);
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function chat_bwj ($aseco, $command) {
	global $bwj_config;


	// Bail out if call did not come from MasterAdmin
	if ( !$aseco->isMasterAdmin($command['author']) ) {
		return;
	}

	// Init
	$message = false;

	if (strtoupper($command['params']) == 'RELOAD') {
		$aseco->console('[plugin.badwords_judge.php] MasterAdmin '. $command['author']->login .' reloads the configuration.');
		$message = '{#admin}Reloading the configuration "badwords_judge.xml" now.';
		bwj_onSync($aseco);
	}

	// Show message
	if ($message != false) {
		if ( ($bwj_config['MESSAGES_IN_WINDOW'][0] == true) && (function_exists('send_window_message')) ) {
			send_window_message($aseco, $message, $command['author']);
		}
		else {
			$aseco->client->query('ChatSendServerMessageToLogin', $aseco->formatColors($message), $command['author']->login);
		}
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onPlayerConnect
function bwj_onPlayerConnect ($aseco, $player) {
	global $bwj_config;


	// Init Player-Punish state
	if ( !isset($bwj_config['PLAYERS']['STATE'][$player->login]) ) {
		// 0 = clean
		// 1 = warned
		// 2 = punished 1
		// 3 = punished 2
		// 4 = punished 3
		// 5 = punished 4
		// after punish 4 (state 5) the Player gets banned for lifetime
		$bwj_config['PLAYERS']['STATE'][$player->login] = 0;
		$bwj_config['PLAYERS']['PUNISH'][$player->login] = false;
	}
	else {
		// Check for open Penalty and try to get them
		if ($bwj_config['PLAYERS']['PUNISH'][$player->login] != false) {

			// $bwj_config['PLAYERS']['PUNISH'][$player->login][0] = coppers
			// $bwj_config['PLAYERS']['PUNISH'][$player->login][1] = penalty
			bwj_collectPenalty($player, $bwj_config['PLAYERS']['PUNISH'][$player->login][0], $bwj_config['PLAYERS']['PUNISH'][$player->login][1]);
		}
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onChat
// [0]=PlayerUid, [1]=Login, [2]=Text, [3]=IsRegistredCmd
function bwj_onChat ($aseco, $chat) {
	global $bwj_config;


	// If server message, bail out immediately
	if ($chat[0] == $aseco->server->id) {
		return;
	}

	// If registered command, bail out immediately
	if ($chat[3] == true) {
		return;
	}

	// If no badwords checking, bail out immediately
	if ($bwj_config['ENABLED'][0] == false) {
		return;
	}


	// Check ignore list
	$player = $aseco->server->players->getPlayer($chat[1]);
	if ( ($bwj_config['IGNORE'][0]['OPERATOR'][0] == true) && ($aseco->isOperator($player)) ) {
		return;
	}
	if ( ($bwj_config['IGNORE'][0]['ADMIN'][0] == true) && ($aseco->isAdmin($player)) ) {
		return;
	}
	if ( ($bwj_config['IGNORE'][0]['MASTERADMIN'][0] == true) && ($aseco->isMasterAdmin($player)) ) {
		return;
	}


	// Prepare what the user entered into the chat
	$words = $chat[2];

	// Remove all signs that can be used as seperator to match also "f.u.c.k" as "fuck"
	$words = preg_replace('/[\W_]+/i', '', $words);

	// Remove more then once occurred signs, e.g. "Shiiit" to "Shit"
	$words = preg_replace('/(.)\\1+/i', '$1', $words);

	if ( (preg_match($bwj_config['PATTERN'], $words, $match)) || (preg_match($bwj_config['PATTERN'], $chat[2], $match)) ) {

		$player = $aseco->server->players->getPlayer($chat[1]);

		// Increase Player state
		$bwj_config['PLAYERS']['STATE'][$player->login] ++;

		$message = false;
		switch ($bwj_config['PLAYERS']['STATE'][$player->login]) {

			// Warning
			case 1:
				$message = formatText($bwj_config['NAME'][0].$bwj_config['MESSAGES'][0]['BADWORD_WARNING'][0],
					'WARNING (1/5):',
					 stripColors($player->nickname),
					$match[0]
				);
				break;

			// Penalty 1
			case 2:
				$message = formatText($bwj_config['NAME'][0].$bwj_config['MESSAGES'][0]['BADWORD_PUNISH'][0],
					'PENALTY (2/5):',
					stripColors($player->nickname),
					$bwj_config['PUNISH_TICKETS'][0]['FINE1'][0],
					$match[0]
				);

				// Display the Penalty
				$penalty = formatText($bwj_config['MESSAGES'][0]['BADWORD_PUNISH_NOTE'][0]);

				// Store punish for later use
				$bwj_config['PLAYERS']['PUNISH'][$player->login] = array($bwj_config['PUNISH_TICKETS'][0]['FINE1'][0], $penalty);

				// Create the ticket
				bwj_collectPenalty($player, $bwj_config['PUNISH_TICKETS'][0]['FINE1'][0], $penalty);
				break;

			// Penalty 2
			case 3:
				$message = formatText($bwj_config['NAME'][0].$bwj_config['MESSAGES'][0]['BADWORD_PUNISH'][0],
					'PENALTY (3/5):',
					stripColors($player->nickname),
					$bwj_config['PUNISH_TICKETS'][0]['FINE2'][0],
					$match[0]
				);

				// Display the Penalty
				$penalty = formatText($bwj_config['MESSAGES'][0]['BADWORD_PUNISH_NOTE'][0]);

				// Store punish for later use
				$bwj_config['PLAYERS']['PUNISH'][$player->login] = array($bwj_config['PUNISH_TICKETS'][0]['FINE2'][0], $penalty);

				// Create the ticket
				bwj_collectPenalty($player, $bwj_config['PUNISH_TICKETS'][0]['FINE2'][0], $penalty);
				break;

			// Penalty 3
			case 4:
				$message = formatText($bwj_config['NAME'][0].$bwj_config['MESSAGES'][0]['BADWORD_PUNISH'][0],
					'PENALTY (4/5):',
					stripColors($player->nickname),
					$bwj_config['PUNISH_TICKETS'][0]['FINE3'][0],
					$match[0]
				);

				// Display the Penalty
				$penalty = formatText($bwj_config['MESSAGES'][0]['BADWORD_PUNISH_NOTE'][0]);

				// Store punish for later use
				$bwj_config['PLAYERS']['PUNISH'][$player->login] = array($bwj_config['PUNISH_TICKETS'][0]['FINE3'][0], $penalty);

				// Create the ticket
				bwj_collectPenalty($player, $bwj_config['PUNISH_TICKETS'][0]['FINE3'][0], $penalty);
				break;

			// Penalty 4
			case 5:
				$message = formatText($bwj_config['NAME'][0].$bwj_config['MESSAGES'][0]['BADWORD_PUNISH'][0],
					'PENALTY (5/5):',
					stripColors($player->nickname),
					$bwj_config['PUNISH_TICKETS'][0]['FINE4'][0],
					$match[0]
				);

				// Display the Penalty
				$penalty = formatText($bwj_config['MESSAGES'][0]['BADWORD_PUNISH_NOTE'][0]);

				bwj_collectPenalty($player, $bwj_config['PUNISH_TICKETS'][0]['FINE4'][0], $penalty);
				break;

			// BAN!
			case 6:
				$message = formatText($bwj_config['NAME'][0].$bwj_config['MESSAGES'][0]['BADWORD_BAN'][0],
					'BAN:',
					stripColors($player->nickname)
				);

				// Ban the Player
				$aseco->client->query('Ban', $player->login, $aseco->formatColors($message));
				break;

			// Do nothing
			default:
				break;
		}


		if ( ($bwj_config['MESSAGES_IN_WINDOW'][0] == true) && (function_exists('send_window_message')) ) {
			send_window_message($aseco, $message, false);
		}
		else {
			$aseco->client->query('ChatSendServerMessage', $aseco->formatColors($message));
		}
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onBillUpdated
// [0]=BillId, [1]=State, [2]=StateName, [3]=TransactionId
function bwj_onBillUpdated ($aseco, $bill) {
	global $bwj_config;


	// Check for known bill ID
	$billid = $bill[0];
	if (array_key_exists($billid, $bwj_config['PLAYERS']['BILLS'])) {

		// Get bill info
		$login		= $bwj_config['PLAYERS']['BILLS'][$billid][0];
		$nickname	= $bwj_config['PLAYERS']['BILLS'][$billid][1];
		$coppers	= $bwj_config['PLAYERS']['BILLS'][$billid][2];

		$message = false;
		switch($bill[1]) {
			// Payed (Paid)
			case 4:
				$aseco->console('[plugin.badwords_judge.php] Player "'. $login .'" paid the Punish of '. $coppers .' Coppers ('. $billid.').');

				$message = formatText($bwj_config['NAME'][0].$bwj_config['MESSAGES'][0]['PUNISH_RECEIVED'][0],
					stripColors($nickname),
					$coppers
				);
				$message = $aseco->formatColors($message);

				// Set punish to false, because Player has paid
				$bwj_config['PLAYERS']['PUNISH'][$login] = false;

				// Remove Bill Id
				unset($bwj_config['PLAYERS']['BILLS'][$billid]);
				break;

			// Refused, kick Player
			case 5:
				$aseco->console('[plugin.badwords_judge.php] Kick Player "'. $login .'", because she/he did not paid the Punish ('. $billid.').');

				$message = formatText($bwj_config['NAME'][0].$bwj_config['MESSAGES'][0]['PUNISH_REFUSED'][0],
					stripColors($nickname),
					$coppers
				);
				$message = $aseco->formatColors($message);

				// Kick the Player
				$aseco->client->query('Kick', $login, $message);

				// Remove Bill Id
				unset($bwj_config['PLAYERS']['BILLS'][$billid]);
				break;

			// Error, remove Payment
			case 6:
				$aseco->console('[plugin.badwords_judge.php] Punish transaction error for Bill ID "'. $billid .'", dropping open Bill ('. $billid.').');

				// Remove Bill Id
				unset($bwj_config['PLAYERS']['BILLS'][$billid]);
				break;

			// Do nothing on:
			// 1 = CreatingTransaction
			// 2 = Issued
			// 3 = ValidatingPay(e)ment
			default:
//				$aseco->console('[plugin.badwords_judge.php] Punish was set to state "'. $bill[2] .'" ('. $billid.').');
				break;
		}

		if ($message != false) {
			if ( ($bwj_config['MESSAGES_IN_WINDOW'][0] == true) && (function_exists('send_window_message')) ) {
				send_window_message($aseco, $message, false);
			}
			else {
				$aseco->client->query('ChatSendServerMessage', $message);
			}
		}
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function bwj_collectPenalty ($player, $coppers, $message) {
	global $aseco, $bwj_config;


	$message = str_replace(
		array(
			'{br}',
		),
		array(
			LF,
		),
		$message
	);

	$aseco->client->resetError();
	$aseco->client->query('SendBill', $player->login, $coppers, $message, '');

	if ( $aseco->client->isError() ) {
		$aseco->console('[plugin.badwords_judge.php] Could not send the bill for Player "'. $player->login .'": ['. $aseco->client->getErrorCode() .'] '. $aseco->client->getErrorMessage());
	}
	else {
		$billid = $aseco->client->getResponse();
		$bwj_config['PLAYERS']['BILLS'][$billid] = array($player->login, $player->nickname, $coppers);
	}
}

?>
